@@ -1,8 +1,8 @@
 import binascii
 
 from django.contrib.auth import authenticate
-from django.conf import settings
 
+from .settings import get_setting
 from .http import JSONResponse
 
 
@@ -20,14 +20,16 @@ def api_authentication_required(func):
     and authenticate the request user appropriately.
     """
     def wrapper(request, *args, **kwargs):
-        AUTH_TYPE = getattr(settings, 'IOS_NOTIFICATIONS_AUTHENTICATION', None)
-        if AUTH_TYPE is None or AUTH_TYPE not in VALID_AUTH_TYPES:
+        AUTH_TYPE = get_setting('IOS_NOTIFICATIONS_AUTHENTICATION')
+        if AUTH_TYPE not in VALID_AUTH_TYPES:
             raise InvalidAuthenticationType('IOS_NOTIFICATIONS_AUTHENTICATION must be specified in your settings.py file.\
                     Valid options are "AuthBasic", "AuthBasicIsStaff" or "AuthNone"')
         # Basic Authorization
         elif AUTH_TYPE == 'AuthBasic' or AUTH_TYPE == 'AuthBasicIsStaff':
             if 'HTTP_AUTHORIZATION' in request.META:
                 auth_type, encoded_user_password = request.META['HTTP_AUTHORIZATION'].split(' ')
+                if (auth_type != 'Basic'):
+                    return JSONResponse({'error': 'Invalid Authorization header, expected Basic'}, status=401)
                 try:
                     userpass = encoded_user_password.decode('base64')
                 except binascii.Error:
