@@ -2,6 +2,7 @@
 
 from optparse import make_option
 import json
+import sys
 
 from django.core.management.base import BaseCommand, CommandError
 
@@ -40,6 +41,10 @@ class Command(BaseCommand):
                     help='Prevent saving the notification in the database after pushing it.',
                     action='store_false',
                     dest='persist'),  # Note: same dest as --persist; they are mutually exclusive
+        make_option('--batch-size',
+                    help='Notifications are sent to devices in batches via the APN Service. This controls the batch size. Default is 100.',
+                    dest='chunk_size',
+                    default=100),
     )
 
     def handle(self, *args, **options):
@@ -59,6 +64,12 @@ class Command(BaseCommand):
         except APNService.DoesNotExist:
             raise CommandError('APNService with id %d does not exist' % service_id)
 
+        message = options['message']
+        extra = options['extra']
+
+        if not message and not extra:
+            raise CommandError('To send a notification you must provide either the --message or --extra option.')
+
         notification = Notification(message=options['message'],
                                     badge=options['badge'],
                                     service=service,
@@ -67,12 +78,17 @@ class Command(BaseCommand):
         if options['persist'] is not None:
             notification.persist = options['persist']
 
-        extra = options['extra']
         if extra is not None:
             notification.extra = json.loads(extra)
 
+        try:
+            chunk_size = int(options['chunk_size'])
+        except ValueError:
+            raise CommandError('The --batch-size option should be an integer value.')
+
         if not notification.is_valid_length():
             raise CommandError('Notification exceeds the maximum payload length. Try making your message shorter.')
 
-        service.push_notification_to_devices(notification)
-        self.stdout.write('Notification pushed successfully\n')
+        service.push_notification_to_devices(notification, chunk_size=chunk_size)
+        if 'test' not in sys.argv:
+            self.stdout.write('Notification pushed successfully\n')
